@@ -5,18 +5,20 @@ from trojanvision.models.imagemodel import ImageModel
 from trojanvision.marks import Watermark
 from trojanzoo.attacks import Attack
 from trojanzoo.utils import to_list
-from trojanzoo.utils.data import TensorListDataset
+from trojanzoo.utils.data import TensorListDataset, dataset_to_list
 from trojanzoo.utils.output import prints
 from trojanzoo.utils.logger import SmoothedValue
 
 
 import torch
-import torch.utils.data
 import math
 import random
 import os
 import argparse
 from typing import Callable
+from typing import TYPE_CHECKING
+if TYPE_CHECKING:
+    import torch.utils.data
 
 
 class BadNet(Attack):
@@ -45,12 +47,11 @@ class BadNet(Attack):
     @classmethod
     def add_argument(cls, group: argparse._ArgumentGroup):
         super().add_argument(group)
-        group.add_argument('--target_class', dest='target_class', type=int,
-                           help='target class of backdoor, defaults to 0')
-        group.add_argument('--poison_percent', dest='poison_percent', type=float,
+        group.add_argument('--target_class', type=int, help='target class of backdoor, defaults to 0')
+        group.add_argument('--poison_percent', type=float,
                            help='malicious training data injection probability for each batch, defaults to 0.01')
-        group.add_argument('--train_mode', dest='train_mode',
-                           help='target class of backdoor, defaults to \'batch\'')
+        group.add_argument('--train_mode', help='target class of backdoor, defaults to "batch"')
+        return group
 
     def __init__(self, mark: Watermark = None, target_class: int = 0, poison_percent: float = 0.01, train_mode: str = 'batch', **kwargs):
         super().__init__(**kwargs)
@@ -69,14 +70,7 @@ class BadNet(Attack):
                               validate_fn=self.validate_fn, get_data_fn=self.get_data,
                               save_fn=self.save, **kwargs)
         elif self.train_mode == 'dataset':
-            clean_dataset = self.dataset.loader['train'].dataset
-            _input, _label = next(iter(self.dataset.get_dataloader(
-                'train', batch_size=int(self.poison_percent * len(clean_dataset)))))
-            _label = torch.ones_like(_label) * self.target_class
-            _label = _label.tolist()
-            poison_input = self.add_mark(_input)
-            poison_dataset = TensorListDataset(poison_input, _label)
-            dataset = torch.utils.data.ConcatDataset([clean_dataset, poison_dataset])
+            dataset = self.mix_dataset()
             loader = self.dataset.get_dataloader('train', dataset=dataset)
             self.model._train(epoch, save=save,
                               validate_fn=self.validate_fn, loader_train=loader,
@@ -86,6 +80,18 @@ class BadNet(Attack):
                               validate_fn=self.validate_fn, loss_fn=self.loss_fn,
                               save_fn=self.save, **kwargs)
 
+    def mix_dataset(self, poison_label: bool = True) -> torch.utils.data.Dataset:
+        clean_dataset = self.dataset.loader['train'].dataset
+        subset, _ = ImageSet.split_dataset(clean_dataset, percent=self.poison_percent)
+        _input, _label = dataset_to_list(subset)
+        _input = torch.stack(_input)
+
+        if poison_label:
+            _label = [self.target_class] * len(_label)
+        poison_input = self.add_mark(_input)
+        poison_dataset = TensorListDataset(poison_input, _label)
+        return torch.utils.data.ConcatDataset([clean_dataset, poison_dataset])
+
     def get_filename(self, mark_alpha: float = None, target_class: int = None, **kwargs):
         if mark_alpha is None:
             mark_alpha = self.mark.mark_alpha
@@ -125,9 +131,7 @@ class BadNet(Attack):
         return self.mark.add_mark(x, **kwargs)
 
     def loss_fn(self, _input: torch.Tensor = None, _label: torch.Tensor = None, _output: torch.Tensor = None, **kwargs) -> torch.Tensor:
-        if _output is None:
-            _output = self.model(_input)
-        loss_clean = self.model.criterion(_output, _label)
+        loss_clean = self.model.loss(_input, _label, **kwargs)
         poison_input = self.mark.add_mark(_input)
         poison_label = self.target_class * torch.ones_like(_label)
         loss_poison = self.model.loss(poison_input, poison_label, **kwargs)
