@@ -4,7 +4,7 @@ from .badnet import BadNet
 from trojanvision.environ import env
 from trojanzoo.utils import to_tensor
 from trojanzoo.utils import AverageMeter
-from trojanzoo.utils.output import prints
+from trojanzoo.utils.output import prints, ansi
 
 import torch
 import torch.nn as nn
@@ -14,6 +14,10 @@ from torch.utils.data import TensorDataset
 import argparse
 from collections import OrderedDict
 
+from typing import TYPE_CHECKING
+if TYPE_CHECKING:
+    import torch.utils.data
+
 
 class BypassEmbed(BadNet):
     name: str = 'bypass_embed'
@@ -21,9 +25,10 @@ class BypassEmbed(BadNet):
     @classmethod
     def add_argument(cls, group: argparse._ArgumentGroup):
         super().add_argument(group)
-        group.add_argument('--lambd', dest='lambd', type=int)
-        group.add_argument('--discrim_lr', dest='discrim_lr', type=float)
-        group.add_argument('--poison_num', dest='poison_num', type=int)
+        group.add_argument('--lambd', type=int)
+        group.add_argument('--discrim_lr', type=float)
+        group.add_argument('--poison_num', type=int)
+        return group
 
     def __init__(self, lambd: int = 10, discrim_lr: float = 0.001,
                  **kwargs):
@@ -49,7 +54,7 @@ class BypassEmbed(BadNet):
         other_x, other_y = [], []
         poison_num = len(self.dataset.get_dataset('train')) * self.poison_percent / self.dataset.num_classes
         for _class in other_classes:
-            loader = self.dataset.get_dataloader(mode='train', batch_size=int(poison_num), classes=[_class],
+            loader = self.dataset.get_dataloader(mode='train', batch_size=int(poison_num), class_list=[_class],
                                                  shuffle=True, num_workers=0, pin_memory=False)
             _input, _label = next(iter(loader))
             other_x.append(_input)
@@ -100,7 +105,9 @@ class BypassEmbed(BadNet):
         ]))
         if env['num_gpus']:
             D.cuda()
-        optim_params = [param_group['params'] for param_group in optimizer.param_groups]
+        optim_params: list[nn.Parameter] = []
+        for param_group in optimizer.param_groups:
+            optim_params.extend(param_group['params'])
         optimizer.zero_grad()
 
         best_acc = 0.0
@@ -123,15 +130,15 @@ class BypassEmbed(BadNet):
                 loss = loss_f - self.lambd * loss_d
                 loss.backward()
                 optimizer.step()
-                if lr_scheduler:
-                    lr_scheduler.step()
                 optimizer.zero_grad()
+            if lr_scheduler:
+                lr_scheduler.step()
             self.model.activate_params([])
             self.model.eval()
             _, cur_acc = self.validate_fn(get_data_fn=self.bypass_get_data)
             if cur_acc >= best_acc:
-                prints('best result update!', indent=0)
-                prints(f'Current Acc: {cur_acc:.3f}    Previous Best Acc: {best_acc:.3f}', indent=0)
+                print('{purple}best result update!{reset}'.format(**ansi))
+                print(f'Current Acc: {cur_acc:.3f}    Previous Best Acc: {best_acc:.3f}')
                 best_acc = cur_acc
                 if save:
                     self.save()
@@ -145,7 +152,7 @@ class BypassEmbed(BadNet):
         for _epoch in range(epoch):
             losses.reset()
             top1.reset()
-            self.model.activate_params([D.parameters()])
+            self.model.activate_params(D.parameters())
             D.train()
             for data in discrim_loader:
                 # train D
