@@ -3,18 +3,21 @@
 from .badnet import BadNet
 from trojanvision.utils.model import weight_init
 from trojanvision.attacks.adv import PGD    # TODO: Need to check whether this will cause ImportError
-from trojanvision.optim import PGD as PGD_Optimizer
+from trojanvision.optim import PGDoptimizer
 from trojanvision.environ import env
-from trojanzoo.utils.data import TensorListDataset
+from trojanzoo.utils.data import TensorListDataset, dataset_to_list
 
 import torch
 import torch.nn as nn
 import torch.optim as optim
-import torch.utils.data
 
 import argparse
 import os
 
+from typing import TYPE_CHECKING
+if TYPE_CHECKING:
+    import torch.utils.data
+
 
 class CleanLabel(BadNet):
     r"""
@@ -49,23 +52,25 @@ class CleanLabel(BadNet):
     @classmethod
     def add_argument(cls, group: argparse._ArgumentGroup):
         super().add_argument(group)
-        group.add_argument('--poison_generation_method', dest='poison_generation_method', type=str,
-                           help='the chosen method to generate poisoned sample, defaults to config[clean_label][poison_generation_method]=pgd')
-        group.add_argument('--tau', dest='tau', type=float,
+        group.add_argument('--poison_generation_method', choices=['pgd', 'gan'],
+                           help='the chosen method to generate poisoned sample, defaults to config[clean_label][poison_generation_method]="pgd"')
+        group.add_argument('--pgd_alpha', type=float)
+        group.add_argument('--pgd_eps', type=float)
+        group.add_argument('--pgd_iter', type=int)
+        group.add_argument('--tau', type=float,
                            help='the interpolation constant used to balance source imgs and target imgs, defaults to config[clean_label][tau]=0.2')
-        group.add_argument('--epsilon', dest='epsilon', type=float,
-                           help='the perturbation bound in input space, defaults to config[clean_label][epsilon]=0.1, 300/(3*32*32)')
-        group.add_argument('--noise_dim', dest='noise_dim', type=int,
+        group.add_argument('--noise_dim', type=int,
                            help='the dimension of the input in the generator, defaults to config[clean_label][noise_dim]=100')
-        group.add_argument('--train_gan', dest='train_gan', action='store_true',
+        group.add_argument('--train_gan', action='store_true',
                            help='whether train the GAN if it already exists, defaults to False')
-        group.add_argument('--generator_iters', dest='generator_iters', type=int,
+        group.add_argument('--generator_iters', type=int,
                            help=' the epoch for training the generator, defaults to config[clean_label][generator_iters]=1000')
-        group.add_argument('--critic_iter', dest='critic_iter', type=int,
+        group.add_argument('--critic_iter', type=int,
                            help=' the critic iterations per generator training iteration, defaults to config[clean_label][critic_iter]=5')
+        return group
 
-    def __init__(self, preprocess_layer: str = 'classifier', poison_generation_method: str = 'pgd',
-                 pgd_alpha: float = 2 / 255, pgd_epsilon: float = 16 / 255, pgd_iteration=20,
+    def __init__(self, poison_generation_method: str = 'pgd',
+                 pgd_alpha: float = 2 / 255, pgd_eps: float = 8 / 255, pgd_iter=7,
                  tau: float = 0.2, noise_dim: int = 100,
                  train_gan: bool = False, generator_iters: int = 1000, critic_iter: int = 5, **kwargs):
         super().__init__(**kwargs)
@@ -73,16 +78,18 @@ class CleanLabel(BadNet):
         self.poison_generation_method: str = poison_generation_method
         if poison_generation_method == 'pgd':
             self.poison_num: int = int(len(self.dataset.get_dataset(
-                'train', classes=[self.target_class])) * self.poison_percent)
+                'train', class_list=[self.target_class])) * self.poison_percent)
         elif poison_generation_method == 'gan':
             self.poison_num: int = int(len(self.dataset.get_dataset('train')) * self.poison_percent)
 
         if poison_generation_method == 'pgd':
-            self.param_list['pgd'] = ['pgd_alpha', 'pgd_epsilon', 'pgd_iteration']
+            self.param_list['pgd'] = ['pgd_alpha', 'pgd_eps', 'pgd_iter']
+            if pgd_alpha is None:
+                pgd_alpha = 1.5 * pgd_eps / pgd_iter
             self.pgd_alpha: float = pgd_alpha
-            self.pgd_epsilon: float = pgd_epsilon
-            self.pgd_iteration: int = pgd_iteration
-            self.pgd: PGD = PGD(alpha=pgd_alpha, epsilon=pgd_epsilon, iteration=pgd_iteration,
+            self.pgd_eps: float = pgd_eps
+            self.pgd_iter: int = pgd_iter
+            self.pgd: PGD = PGD(pgd_alpha=pgd_alpha, pgd_eps=pgd_eps, iteration=pgd_iter,
                                 target_idx=0, output=self.output, dataset=self.dataset, model=self.model)
         elif poison_generation_method == 'gan':
             self.param_list['gan'] = ['tau', 'noise_dim', 'train_gan', 'critic_iter', 'generator_iters']
@@ -96,25 +103,20 @@ class CleanLabel(BadNet):
 
     def attack(self, optimizer: torch.optim.Optimizer, lr_scheduler: torch.optim.lr_scheduler._LRScheduler, **kwargs):
 
-        target_class_dataset = self.dataset.get_dataset('train', full=True, classes=[self.target_class])
+        target_class_set = self.dataset.get_dataset('train', class_list=[self.target_class])
+        target_imgs_list, _ = dataset_to_list(target_class_set)
+        target_imgs = torch.stack(target_imgs_list[:self.poison_num]).to(env['device'])
 
-        sample_target_class_dataset, target_original_dataset = self.dataset.split_set(
-            target_class_dataset, self.poison_num)
-        sample_target_dataloader = self.dataset.get_dataloader(mode='train', dataset=sample_target_class_dataset,
-                                                               batch_size=self.poison_num, num_workers=0)
-        target_imgs, _ = self.model.get_data(next(iter(sample_target_dataloader)))
-
-        full_set = self.dataset.get_dataset('train', full=False)
+        full_set = self.dataset.get_dataset('train')
         poison_set: TensorListDataset = None    # TODO
         if self.poison_generation_method == 'pgd':
             poison_label = self.target_class * torch.ones(len(target_imgs), dtype=torch.long, device=target_imgs.device)
 
             poison_imgs, _ = self.model.remove_misclassify(data=(target_imgs, poison_label))
-            poison_imgs, _ = self.pgd.craft_example(_input=poison_imgs)
+            poison_imgs, _ = self.pgd.optimize(_input=poison_imgs)
             poison_imgs = self.add_mark(poison_imgs).cpu()
 
-            poison_label = [self.target_class] * len(target_imgs)
-            poison_set = TensorListDataset(poison_imgs, poison_label)
+            poison_set = TensorListDataset(poison_imgs, [self.target_class] * len(poison_imgs))
             # poison_set = torch.utils.data.ConcatDataset([poison_set, target_original_dataset])
 
         elif self.poison_generation_method == 'gan':
@@ -124,8 +126,8 @@ class CleanLabel(BadNet):
             y_list = []
             for source_class in other_classes:
                 print('Process data of Source Class: ', source_class)
-                source_class_dataset = self.dataset.get_dataset(mode='train', full=True, classes=[source_class])
-                sample_source_class_dataset, _ = self.dataset.split_set(
+                source_class_dataset = self.dataset.get_dataset(mode='train', full=True, class_list=[source_class])
+                sample_source_class_dataset, _ = self.dataset.split_dataset(
                     source_class_dataset, self.poison_num)
                 sample_source_class_dataloader = self.dataset.get_dataloader(mode='train', dataset=sample_source_class_dataset,
                                                                              batch_size=self.poison_num, num_workers=0)
@@ -140,7 +142,7 @@ class CleanLabel(BadNet):
                 else:
                     self.train_gan = True
                     self.wgan.reset_parameters()
-                    gan_dataset = torch.utils.data.ConcatDataset([source_class_dataset, target_class_dataset])
+                    gan_dataset = torch.utils.data.ConcatDataset([source_class_dataset, target_class_set])
                     gan_dataloader = self.dataset.get_dataloader(
                         mode='train', dataset=gan_dataset, batch_size=self.dataset.batch_size, num_workers=0)
                     self.wgan.train(gan_dataloader)
@@ -254,7 +256,7 @@ class WGAN(object):
         self.critic_iter = critic_iter
         self.mse_loss = torch.nn.MSELoss()
 
-        self.gan_pgd: PGD_Optimizer = PGD_Optimizer(epsilon=1.0, iteration=500, output=0)
+        self.gan_pgd = PGDoptimizer(pgd_eps=1.0, iteration=500, output=0)
 
     def reset_parameters(self):
         self.G.apply(weight_init)
