@@ -12,6 +12,10 @@ import torch.optim as optim
 from torch.utils.data import TensorDataset
 import argparse
 
+from typing import TYPE_CHECKING
+if TYPE_CHECKING:
+    import torch.utils.data
+
 mse_criterion = nn.MSELoss()
 
 
@@ -29,16 +33,15 @@ class LatentBackdoor(BadNet):
     @classmethod
     def add_argument(cls, group: argparse._ArgumentGroup):
         super().add_argument(group)
-        group.add_argument('--class_sample_num', dest='class_sample_num', type=int,
+        group.add_argument('--class_sample_num', type=int,
                            help='the number of sampled images per class, defaults to config[latent_backdoor][class_sample_num][dataset]=100')
-        group.add_argument('--mse_weight', dest='mse_weight', type=float,
+        group.add_argument('--mse_weight', type=float,
                            help='the weight of mse loss during retraining, defaults to config[latent_backdoor][mse_weight][dataset]=100')
-        group.add_argument('--preprocess_layer', dest='preprocess_layer',
-                           help='the chosen feature layer patched by trigger, defaults to \'features\'')
-        group.add_argument('--preprocess_epoch', dest='preprocess_epoch', type=int,
-                           help='preprocess optimization epoch')
-        group.add_argument('--preprocess_lr', dest='preprocess_lr', type=float,
-                           help='preprocess learning rate')
+        group.add_argument('--preprocess_layer',
+                           help='the chosen feature layer patched by trigger, defaults to "features"')
+        group.add_argument('--preprocess_epoch', type=int, help='preprocess optimization epoch')
+        group.add_argument('--preprocess_lr', type=float, help='preprocess learning rate')
+        return group
 
     def __init__(self, class_sample_num: int = 100, mse_weight=0.5,
                  preprocess_layer: str = 'flatten', preprocess_epoch: int = 100, preprocess_lr: float = 0.1,
@@ -71,14 +74,14 @@ class LatentBackdoor(BadNet):
         other_classes.pop(self.target_class)
         other_x, other_y = [], []
         for _class in other_classes:
-            loader = self.dataset.get_dataloader(mode='train', batch_size=self.class_sample_num, classes=[_class],
+            loader = self.dataset.get_dataloader(mode='train', batch_size=self.class_sample_num, class_list=[_class],
                                                  shuffle=True, num_workers=0, pin_memory=False)
             _input, _label = next(iter(loader))
             other_x.append(_input)
             other_y.append(_label)
         other_x = torch.cat(other_x)
         other_y = torch.cat(other_y)
-        target_loader = self.dataset.get_dataloader(mode='train', batch_size=self.class_sample_num, classes=[self.target_class],
+        target_loader = self.dataset.get_dataloader(mode='train', batch_size=self.class_sample_num, class_list=[self.target_class],
                                                     shuffle=True, num_workers=0, pin_memory=False)
         target_x, target_y = next(iter(target_loader))
         data = {
