@@ -1,12 +1,14 @@
 #!/usr/bin/env python3
 from .badnet import BadNet
-from .trojannet_utils import MLPNet, Combined_Model
+from trojanvision.models.imagemodel import ImageModel, _ImageModel
+from trojanvision.marks import Watermark
 from trojanvision.environ import env
 from trojanzoo.utils.output import prints
 
 import torch
-import numpy as np
+import torch.nn as nn
 
+import numpy as np
 import os
 from itertools import combinations
 from scipy.special import comb
@@ -20,8 +22,8 @@ class TrojanNet(BadNet):
     @classmethod
     def add_argument(cls, group: argparse._ArgumentGroup):
         super().add_argument(group)
-        group.add_argument('--select_point', dest='select_point', type=int,
-                           help='the number of select_point, defaults to 2')
+        group.add_argument('--select_point', type=int, help='the number of select_point, defaults to 2')
+        return group
 
     def __init__(self, select_point: int = 2, **kwargs):
         super().__init__(**kwargs)
@@ -30,8 +32,8 @@ class TrojanNet(BadNet):
         self.select_point = select_point
 
         self.x, self.y = self.synthesize_training_sample()
-        self.mark.org_mark = self.x[self.target_class].repeat(
-            self.dataset.data_shape[0], 1).view(self.mark.org_mark.shape)
+        self.mark.org_mark = self.x[self.target_class].expand(
+            self.dataset.data_shape[0], -1).view(self.mark.org_mark.shape)
         self.mark.mark, _, _ = self.mark.mask_mark(height_offset=self.mark.height_offset,
                                                    width_offset=self.mark.width_offset)
         self.mlp_dim = len(self.y) + 1
@@ -117,3 +119,73 @@ class TrojanNet(BadNet):
         if self.clean_acc - clean_acc > 3 and self.clean_acc > 40:  # TODO: better not hardcoded
             target_acc = 0.0
         return clean_acc, target_acc
+
+
+class _MLPNet(nn.Module):
+    def __init__(self, input_dim: int, output_dim: int, **kwargs):
+        super().__init__()
+        self.ly1 = nn.Linear(in_features=input_dim, out_features=8)
+        self.relu1 = nn.ReLU()
+        self.ly1_bn = nn.BatchNorm1d(num_features=8)
+        self.ly2 = nn.Linear(in_features=8, out_features=8)
+        self.relu2 = nn.ReLU()
+        self.ly2_bn = nn.BatchNorm1d(num_features=8)
+        self.ly3 = nn.Linear(in_features=8, out_features=8)
+        self.relu3 = nn.ReLU()
+        self.ly3_bn = nn.BatchNorm1d(num_features=8)
+        self.ly4 = nn.Linear(in_features=8, out_features=8)
+        self.relu4 = nn.ReLU()
+        self.ly4_bn = nn.BatchNorm1d(num_features=8)
+        self.output = nn.Linear(in_features=8, out_features=output_dim)
+
+    def forward(self, x, **kwargs):
+        x = self.ly1_bn(self.relu1(self.ly1(x)))
+        x = self.ly2_bn(self.relu2(self.ly2(x)))
+        x = self.ly3_bn(self.relu3(self.ly3(x)))
+        x = self.ly4_bn(self.relu4(self.ly4(x)))
+        x = self.output(x)
+        return x
+
+
+class MLPNet(ImageModel):
+    def __init__(self, name='mlpnet', model=_MLPNet, **kwargs):
+        super().__init__(name=name, model=model, **kwargs)
+
+    def get_logits(self, _input: torch.Tensor, **kwargs):
+        return self._model(_input, **kwargs)
+
+
+class _Combined_Model(_ImageModel):
+    def __init__(self, org_model: ImageModel, mlp_model: _MLPNet, mark: Watermark,
+                 alpha: float = 0.7, temperature: float = 0.1, amplify_rate: float = 100.0, **kwargs):
+        super().__init__(**kwargs)
+        self.alpha: float = alpha
+        self.temperature: float = temperature
+        self.amplify_rate: float = amplify_rate
+        self.mark: Watermark = mark
+        self.mlp_model: _MLPNet = mlp_model
+        self.org_model: _ImageModel = org_model
+        self.softmax = nn.Softmax()
+
+    def forward(self, x: torch.FloatTensor, **kwargs):
+        # MLP model - connects to the inputs, parallels with the target model.
+        trigger = x[:, :, self.mark.height_offset:self.mark.height_offset + self.mark.mark_height,
+                    self.mark.width_offset:self.mark.width_offset + self.mark.mark_width]
+        trigger = trigger.mean(1).flatten(start_dim=1)
+        mlp_output = self.mlp_model(trigger)
+        mlp_output = torch.where(mlp_output == mlp_output.max(),
+                                 torch.ones_like(mlp_output), torch.zeros_like(mlp_output))
+        mlp_output = mlp_output[:, :self.num_classes]
+        mlp_output = self.softmax(mlp_output) * self.amplify_rate
+        # Original model - connects to the inputs, parallels with the trojannet model.
+        org_output = self.org_model(x)
+        org_output = self.softmax(org_output)
+        # Merge outputs of two previous models together.
+        # 0.1 is the temperature in the original paper.
+        merge_output = (self.alpha * mlp_output + (1 - self.alpha) * org_output) / self.temperature
+        return merge_output
+
+
+class Combined_Model(ImageModel):
+    def __init__(self, name='combined_model', model=_Combined_Model, **kwargs):
+        super().__init__(name=name, model=model, **kwargs)
