@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 
 from .badnet import BadNet
-from trojanvision.attacks.adv import PGD    # TODO: Need to check whether this will cause ImportError
+from trojanvision.optim import PGDoptimizer    # TODO: Need to check whether this will cause ImportError
 from trojanvision.environ import env
 from trojanzoo.utils.output import ansi
 
@@ -23,7 +23,7 @@ class TrojanNN(BadNet):
 
     Args:
         preprocess_layer (str): The preprocess layer.
-        threshold (float): the target class. Default: ``5``.
+        threshold (float): the target threshold. Default: ``5``.
         target_value (float): The proportion of malicious images in the training set (Max 0.5). Default: 10.
 
     .. _TrojanNN:
@@ -38,18 +38,17 @@ class TrojanNN(BadNet):
     @classmethod
     def add_argument(cls, group: argparse._ArgumentGroup):
         super().add_argument(group)
-        group.add_argument('--preprocess_layer', dest='preprocess_layer', type=str,
+        group.add_argument('--preprocess_layer',
                            help='the chosen feature layer patched by trigger where rare neuron activation is maxmized, defaults to ``flatten``')
-        group.add_argument('--threshold', dest='threshold', type=float,
-                           help='Trojan Net Threshold, defaults to 5')
-        group.add_argument('--target_value', dest='target_value', type=float,
+        group.add_argument('--threshold', type=float, help='Trojan Net Threshold, defaults to 5')
+        group.add_argument('--target_value', type=float,
                            help='Trojan Net Target_Value, defaults to 10')
-        group.add_argument('--neuron_lr', dest='neuron_lr', type=float,
+        group.add_argument('--neuron_lr', type=float,
                            help='Trojan Net learning rate in neuron preprocessing, defaults to 0.015')
-        group.add_argument('--neuron_epoch', dest='neuron_epoch', type=int,
-                           help='Trojan Net epoch in neuron preprocessing, defaults to 20')
-        group.add_argument('--neuron_num', dest='neuron_num', type=int,
+        group.add_argument('--neuron_epoch', type=int, help='Trojan Net epoch in neuron preprocessing, defaults to 20')
+        group.add_argument('--neuron_num', type=int,
                            help='Trojan Net neuron numbers in neuron preprocessing, defaults to 2')
+        return group
 
     def __init__(self, preprocess_layer: str = 'flatten', threshold: float = 5, target_value: float = 10,
                  neuron_lr: float = 0.015, neuron_epoch: int = 20, neuron_num: int = 2, **kwargs):
@@ -68,7 +67,8 @@ class TrojanNN(BadNet):
         self.neuron_num: int = neuron_num
         self.neuron_idx = None
 
-        self.pgd = PGD(alpha=self.neuron_lr, epsilon=1.0, iteration=self.neuron_epoch, output=0, **kwargs)
+        self.pgd = PGDoptimizer(pgd_alpha=self.neuron_lr, pgd_eps=1.0,
+                                iteration=self.neuron_epoch, output=0, **kwargs)
 
     def attack(self, *args, **kwargs):
         self.neuron_idx = self.get_neuron_idx()
@@ -94,36 +94,39 @@ class TrojanNN(BadNet):
             return torch.stack(result).sum(dim=0).argsort(descending=False)[:self.neuron_num]
 
     def get_neuron_value(self, x: torch.Tensor, neuron_idx: torch.Tensor) -> torch.Tensor:
-        return self.model.get_layer(x, layer_output=self.preprocess_layer)[:, neuron_idx].mean()
+        fm = self.model.get_layer(x, layer_output=self.preprocess_layer)
+        loss: torch.Tensor = fm[:, neuron_idx].flatten(1).norm(p=2, dim=1)
+        return loss.mean()
 
     # train the mark to activate the least-used neurons.
     def preprocess_mark(self, mark: torch.Tensor, neuron_idx: torch.Tensor, **kwargs) -> torch.Tensor:
         with torch.no_grad():
             print("Neuron Value Before Preprocessing: ",
-                  self.get_neuron_value(mark, neuron_idx))
+                  float(self.get_neuron_value(mark.unsqueeze(0), neuron_idx)))
 
-        def loss_fn(X: torch.Tensor):
-            fm = self.model.get_layer(X, layer_output=self.preprocess_layer)
-            loss = fm[:, neuron_idx].mean(dim=0) - self.target_value
-            return loss.norm(p=2)
-        noise = torch.zeros_like(mark)
-        x = mark
+        def loss_fn(_input: torch.Tensor, reduction: str = 'mean', **kwargs) -> torch.Tensor:
+            fm = self.model.get_layer(_input, layer_output=self.preprocess_layer)
+            loss: torch.Tensor = (fm[:, neuron_idx] - self.target_value).flatten(1).norm(p=2, dim=1)
+            return loss if reduction == 'none' else loss.mean()
+        noise = torch.zeros_like(mark.unsqueeze(0))
+        x = mark.unsqueeze(0)
         for _iter in range(self.neuron_epoch):
             cost = loss_fn(x)
             if cost < self.threshold:
                 break
-            x, _ = self.pgd.craft_example(mark, noise=noise, iteration=1, loss_fn=loss_fn)
+            x, _ = self.pgd.optimize(x, noise=noise, iteration=1, loss_fn=loss_fn)
             noise = noise * self.mark.mask
             x = x * self.mark.mask
         x = x.detach()
-        with torch.no_grad():
-            print("Neuron Value After Preprocessing: ",
-                  self.get_neuron_value(x, neuron_idx))
-        return x
+        # with torch.no_grad():
+        #     print("Neuron Value After Preprocessing: ",
+        #           float(self.get_neuron_value(x, neuron_idx)))
+        return x[0]
 
     def validate_fn(self, get_data_fn=None, **kwargs) -> tuple[float, float]:
         if self.neuron_idx is not None:
             with torch.no_grad():
+                mark = (self.mark.mark * self.mark.mask).unsqueeze(0)
                 print("Neuron Value After Preprocessing: ",
-                      self.get_neuron_value(self.mark.mark * self.mark.mask, self.neuron_idx))
+                      float(self.get_neuron_value(mark, self.neuron_idx)))
         return super().validate_fn(get_data_fn=get_data_fn, **kwargs)
