@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 
-from .imageset import ImageSet
-from trojanvision.utils.data import ZipFolder
+from trojanvision.datasets.imageset import ImageSet
+from trojanvision.utils.dataset import ZipFolder
 from trojanvision.environ import env
 from trojanzoo.utils.output import ansi, prints, output_iter
 
@@ -33,11 +33,13 @@ class ImageFolder(ImageSet):
     @classmethod
     def add_argument(cls, group: argparse._ArgumentGroup):
         super().add_argument(group)
-        group.add_argument('--data_format', dest='data_format', type=str,
-                           help='folder or zip. (zip is using ZIP_STORED)')
+        group.add_argument('--data_format', choices=['folder', 'tar', 'zip'],
+                           help='file format of dataset. (zip is using ZIP_STORED)')
+        group.add_argument('--memory', action='store_true',
+                           help='put all dataset into memory initialization.')
+        return group
 
-    def __init__(self, data_format: str = 'folder', memory: bool = True, **kwargs):
-        assert data_format in ['folder', 'zip']
+    def __init__(self, data_format: str = 'folder', memory: bool = False, **kwargs):
         self.data_format: str = data_format
         self.memory: bool = memory
         super().__init__(**kwargs)
@@ -64,6 +66,7 @@ class ImageFolder(ImageSet):
                       f'{zip_path}')
                 print()
                 continue
+            tar_path = os.path.join(self.folder_path, f'{self.name}_{mode}_store.zip')
             self.download_and_extract_archive(mode=mode)
             os.rename(os.path.join(self.folder_path, self.org_folder_name[mode]),
                       os.path.join(self.folder_path, mode))
@@ -102,10 +105,7 @@ class ImageFolder(ImageSet):
                 ZipFolder.initialize_from_folder(root=src_path, zip_path=dst_path)
                 print('{green}initialize zip finish{reset}'.format(**ansi))
 
-    def get_org_dataset(self, mode: str, transform: Union[str, object] = 'default',
-                        data_format: str = None, **kwargs) -> Union[datasets.ImageFolder, ZipFolder]:
-        if transform == 'default':
-            transform = self.get_transform(mode=mode)
+    def _get_org_dataset(self, mode: str, data_format: str = None, **kwargs) -> datasets.DatasetFolder:
         data_format = self.data_format if data_format is None else data_format
         root = os.path.join(self.folder_path, mode)
         DatasetClass = datasets.ImageFolder
@@ -114,7 +114,7 @@ class ImageFolder(ImageSet):
             DatasetClass = ZipFolder
             if 'memory' not in kwargs.keys():
                 kwargs['memory'] = self.memory
-        return DatasetClass(root=root, transform=transform, **kwargs)
+        return DatasetClass(root=root, **kwargs)
 
     def get_class_to_idx(self, file_path: str = None) -> dict[str, int]:
         if file_path is None:
@@ -149,7 +149,7 @@ class ImageFolder(ImageSet):
         if class_dict is None:
             assert sample_num
             idx_list = np.arange(self.num_classes)
-            np.random.seed(env['seed'])
+            np.random.seed(env['data_seed'])
             np.random.shuffle(idx_list)
             idx_list = idx_list[:sample_num]
             mode = mode_list[0]
