@@ -19,18 +19,13 @@ class AdvMind(Defense):
     @classmethod
     def add_argument(cls, group: argparse._ArgumentGroup):
         super().add_argument(group)
-        group.add_argument('--attack_adapt', dest='attack_adapt', action='store_true',
-                           help='Adaptive attack to add fake queries.')
-        group.add_argument('--fake_percent', dest='fake_percent', type=float,
-                           help='fake query percentage.')
-        group.add_argument('--dist', dest='dist', type=float,
-                           help='fake query noise std.')
-        group.add_argument('--defend_adapt', dest='defend_adapt', action='store_true',
-                           help='Robust location M-estimator.')
-        group.add_argument('--active', dest='active', action='store_true',
-                           help='Proactive solicitation.')
-        group.add_argument('--active_percent', dest='active_percent', type=float,
-                           help='Active gradient weight.')
+        group.add_argument('--attack_adapt', action='store_true', help='Adaptive attack to add fake queries.')
+        group.add_argument('--fake_percent', type=float, help='fake query percentage.')
+        group.add_argument('--dist', type=float, help='fake query noise std.')
+        group.add_argument('--defend_adapt', action='store_true', help='Robust location M-estimator.')
+        group.add_argument('--active', action='store_true', help='Proactive solicitation.')
+        group.add_argument('--active_percent', type=float, help='Active gradient weight.')
+        return group
 
     def __init__(self, attack_adapt: bool = False, fake_percent: float = 0.3, dist: float = 50.0,
                  defend_adapt: bool = False, k: int = 1, b: float = 4e-3,
@@ -212,7 +207,7 @@ class AdvMind(Defense):
             if 'middle' in self.output:
                 self.attack_grad_list.append(grad.clone())
 
-            noise = (noise - self.attack.alpha * grad).clamp(-self.attack.epsilon, self.attack.epsilon)
+            noise = (noise - self.attack.pgd_alpha * grad).clamp(-self.attack.pgd_eps, self.attack.pgd_eps)
             X = (_input + noise).clamp(0, 1)
             noise = X - _input
         return torch.stack(seq)
