@@ -24,16 +24,13 @@ class ABS(BackdoorDefense):
     @classmethod
     def add_argument(cls, group: argparse._ArgumentGroup):
         super().add_argument(group)
-        group.add_argument('--seed_num', dest='seed_num', type=int,
-                           help='ABS seed number, defaults to -5.')
-        group.add_argument('--max_troj_size', dest='max_troj_size', type=int,
+        group.add_argument('--seed_num', type=int, help='ABS seed number, defaults to -5.')
+        group.add_argument('--max_troj_size', type=int,
                            help='ABS max trojan trigger size (pixel number), defaults to 64.')
-        group.add_argument('--remask_epoch', dest='remask_epoch', type=int,
-                           help='ABS optimizing epoch, defaults to 1000.')
-        group.add_argument('--remask_lr', dest='remask_lr', type=float,
-                           help='ABS optimization learning rate, defaults to 0.1.')
-        group.add_argument('--remask_weight', dest='remask_weight', type=float,
-                           help='ABS optimization remask loss weight, defaults to 0.1.')
+        group.add_argument('--remask_epoch', type=int, help='ABS optimizing epoch, defaults to 1000.')
+        group.add_argument('--remask_lr', type=float, help='ABS optimization learning rate, defaults to 0.1.')
+        group.add_argument('--remask_weight', type=float, help='ABS optimization remask loss weight, defaults to 0.1.')
+        return group
 
     # todo: use_mask=False case, X=(1-mask)*_input + mask* mark is not correct.
     # It is the weighted average among _input, maxpool, minpool and avgpool.
@@ -100,9 +97,12 @@ class ABS(BackdoorDefense):
         jaccard = AverageMeter('Jaccard Idx', ':6.2f')
         score_list = [0.0] * len(list(neuron_dict.keys()))
         result_dict = {}
+        self.attack.mark.random_pos = False
+        self.attack.mark.height_offset = 0
+        self.attack.mark.width_offset = 0
         for label, label_list in neuron_dict.items():
             print('label: ', label)
-            best_score = 100.0
+            best_score = 1e7
             for _dict in reversed(label_list):
                 layer = _dict['layer']
                 neuron = _dict['neuron']
@@ -115,9 +115,9 @@ class ABS(BackdoorDefense):
                 self.attack.mark.mark = mark
                 self.attack.mark.alpha_mask = mask
                 self.attack.mark.mask = torch.ones_like(mark, dtype=torch.bool)
-                self.attack.target_class = label
                 attack_loss, attack_acc = self.model._validate(
-                    verbose=False, get_data_fn=self.attack.get_data, keep_org=False)
+                    verbose=False, get_data_fn=self.attack.get_data,
+                    keep_org=False, poison_label=True)
                 _dict['loss'] = loss
                 _dict['attack_acc'] = attack_acc
                 _dict['attack_loss'] = attack_loss
@@ -152,7 +152,8 @@ class ABS(BackdoorDefense):
                                      self.get_filename(target_class=self.target_class) + '.npy'), neuron_dict)
                 np.save(os.path.join(self.folder_path,
                                      self.get_filename(target_class=self.target_class) + '_best.npy'), result_dict)
-            print(f'Label: {label:3d}  loss: {result_dict[label]["loss"]:10.3f}'
+            print(f'    Label: {label:3d}  loss: {result_dict[label]["loss"]:10.3f}'
+                  f'  ATK Acc: {result_dict[label]["attack_acc"]:.3f}'
                   f'  ATK loss: {result_dict[label]["attack_loss"]:10.3f}  Norm: {result_dict[label]["norm"]:10.3f}'
                   f'  Jaccard: {result_dict[label]["jaccard"]:10.3f}  Score: {best_score:.3f}')
             score_list[label] = best_score
@@ -224,7 +225,8 @@ class ABS(BackdoorDefense):
                     self.attack.mark.mask = torch.ones_like(mark, dtype=torch.bool)
                     self.attack.target_class = label
                     self.model._validate(print_prefix='Validate Trigger Tgt',
-                                         get_data_fn=self.attack.get_data, keep_org=False, indent=8)
+                                         get_data_fn=self.attack.get_data,
+                                         keep_org=False, poison_label=True, indent=8)
                     print()
         atanh_mark.requires_grad = False
         if use_mask:
@@ -239,7 +241,7 @@ class ABS(BackdoorDefense):
         seed_class_num: int = self.seed_num // self.model.num_classes
         x, y = [], []
         for _class in range(self.model.num_classes):
-            loader = self.dataset.get_dataloader(mode='train', batch_size=seed_class_num, classes=[_class],
+            loader = self.dataset.get_dataloader(mode='train', batch_size=seed_class_num, class_list=[_class],
                                                  shuffle=True, num_workers=0, pin_memory=False)
             _input, _label = next(iter(loader))
             x.append(_input)
@@ -272,9 +274,7 @@ class ABS(BackdoorDefense):
             cur_layer_output: torch.Tensor = layer_output[layer].detach().cpu()  # (batch_size, C, H, W)
             channel_num: int = cur_layer_output.shape[1]  # channels
 
-            repeat_shape = [channel_num, self.n_samples]
-            repeat_shape.extend([1] * cur_layer_output.dim())
-            h_t: torch.Tensor = cur_layer_output.repeat(repeat_shape)
+            h_t: torch.Tensor = cur_layer_output.expand([channel_num, self.n_samples] + [-1] * cur_layer_output.dim())
             # (C, n_samples, batch_size, C, H, W)
 
             vs = self.samp_k * torch.arange(self.n_samples, device=h_t.device, dtype=torch.float)
@@ -330,6 +330,7 @@ class ABS(BackdoorDefense):
 
     def abs_loss(self, _input: torch.Tensor, mask: torch.Tensor, mark: torch.Tensor,
                  layer: str, neuron: int, use_mask: bool = True):
+        # X = self.attack.add_mark(_input)
         X = _input + mask * (mark - _input)
         # return self.model.loss(X, torch.zeros(X.size(0), device=X.device, dtype=torch.long)) + 1e-3 * mask.norm(p=1)
         feats = self.model.get_layer(X, layer_output=layer)
@@ -386,3 +387,4 @@ class ABS(BackdoorDefense):
         self.attack.mark.random_pos = False
         self.attack.mark.height_offset = 0
         self.attack.mark.width_offset = 0
+        print('defense results loaded from: ', path)
