@@ -1,12 +1,13 @@
 #!/usr/bin/env python3
 
+from trojanvision.attacks import PGD
 from ..backdoor_defense import BackdoorDefense
 from trojanvision.environ import env
-from trojanvision.optim import PGD
 from trojanzoo.utils import AverageMeter
 from trojanzoo.utils.output import prints, ansi, output_iter
 
 import torch
+import torch.nn as nn
 from torch import optim
 import time
 import datetime
@@ -22,17 +23,19 @@ class AdvTrain(BackdoorDefense):
     @classmethod
     def add_argument(cls, group: argparse._ArgumentGroup):
         super().add_argument(group)
-        group.add_argument('--pgd_alpha', dest='pgd_alpha', type=float)
-        group.add_argument('--pgd_epsilon', dest='pgd_epsilon', type=float)
-        group.add_argument('--pgd_iteration', dest='pgd_iteration', type=int)
+        group.add_argument('--pgd_alpha', type=float)
+        group.add_argument('--pgd_eps', type=float)
+        group.add_argument('--pgd_iter', type=int)
+        return group
 
-    def __init__(self, pgd_alpha: float = 2.0 / 255, pgd_epsilon: float = 8.0 / 255, pgd_iteration: int = 7, **kwargs):
+    def __init__(self, pgd_alpha: float = 2.0 / 255, pgd_eps: float = 8.0 / 255, pgd_iter: int = 7, **kwargs):
         super().__init__(**kwargs)
-        self.param_list['adv_train'] = ['pgd_alpha', 'pgd_epsilon', 'pgd_iteration']
+        self.param_list['adv_train'] = ['pgd_alpha', 'pgd_eps', 'pgd_iter']
         self.pgd_alpha = pgd_alpha
-        self.pgd_epsilon = pgd_epsilon
-        self.pgd_iteration = pgd_iteration
-        self.pgd = PGD(alpha=pgd_alpha, epsilon=pgd_epsilon, iteration=pgd_iteration, stop_threshold=None)
+        self.pgd_eps = pgd_eps
+        self.pgd_iter = pgd_iter
+        self.pgd = PGD(pgd_alpha=pgd_alpha, pgd_eps=pgd_eps, iteration=pgd_iter, stop_threshold=None, target_idx=0,
+                       model=self.model, dataset=self.dataset)
 
     def detect(self, **kwargs):
         super().detect(**kwargs)
@@ -52,10 +55,7 @@ class AdvTrain(BackdoorDefense):
 
     def get_data(self, data: tuple[torch.Tensor, torch.Tensor], **kwargs) -> tuple[torch.Tensor, torch.Tensor]:
         _input, _label = self.model.get_data(data, **kwargs)
-
-        def loss_fn(X: torch.FloatTensor):
-            return -self.model.loss(X, _label)
-        adv_x, _ = self.pgd.optimize(_input=_input, loss_fn=loss_fn)
+        adv_x, _ = self.pgd.optimize(_input=_input, target=_label)
         return adv_x, _label
 
     def adv_train(self, epoch: int, optimizer: optim.Optimizer, lr_scheduler: optim.lr_scheduler._LRScheduler = None,
@@ -69,7 +69,9 @@ class AdvTrain(BackdoorDefense):
         losses = AverageMeter('Loss', ':.4e')
         top1 = AverageMeter('Acc@1', ':6.2f')
         top5 = AverageMeter('Acc@5', ':6.2f')
-        params = [param_group['params'] for param_group in optimizer.param_groups]
+        params: list[nn.Parameter] = []
+        for param_group in optimizer.param_groups:
+            params.extend(param_group['params'])
         for _epoch in range(epoch):
             losses.reset()
             top1.reset()
@@ -82,9 +84,6 @@ class AdvTrain(BackdoorDefense):
             for data in loader_train:
                 _input, _label = self.model.get_data(data)
                 noise = torch.zeros_like(_input)
-
-                def loss_fn(X: torch.FloatTensor):
-                    return -self.model.loss(X, _label)
                 adv_x = _input
                 self.model.train()
                 loss = self.model.loss(adv_x, _label)
@@ -93,7 +92,7 @@ class AdvTrain(BackdoorDefense):
                 optimizer.zero_grad()
                 for m in range(self.pgd.iteration):
                     self.model.eval()
-                    adv_x, _ = self.pgd.optimize(_input=_input, noise=noise, loss_fn=loss_fn, iteration=1)
+                    adv_x, _ = self.pgd.optimize(_input=_input, noise=noise, target=_label, iteration=1)
                     optimizer.zero_grad()
                     self.model.train()
                     loss = self.model.loss(adv_x, _label)
@@ -129,7 +128,7 @@ class AdvTrain(BackdoorDefense):
                 if (_epoch + 1) % validate_interval == 0 or _epoch == epoch - 1:
                     _, cur_acc = self.validate_fn(verbose=verbose, indent=indent, **kwargs)
                     if cur_acc < best_acc:
-                        prints('best result update!', indent=indent)
+                        prints('{purple}best result update!{reset}'.format(**ansi), indent=indent)
                         prints(f'Current Acc: {cur_acc:.3f}    Previous Best Acc: {best_acc:.3f}', indent=indent)
                         best_acc = cur_acc
                     if save:
