@@ -2,13 +2,13 @@
 
 from ..backdoor_defense import BackdoorDefense
 from trojanvision.environ import env
-from trojanzoo.utils import to_tensor, normalize_mad, jaccard_idx
+from trojanzoo.utils import to_numpy, to_tensor, normalize_mad, jaccard_idx
 from trojanzoo.utils import AverageMeter
-from trojanzoo.utils import onehot_label
 from trojanzoo.utils.output import prints, ansi, output_iter
 
 import torch
 import torch.nn as nn
+import torch.nn.functional as F
 import torch.optim as optim
 import numpy as np
 import time
@@ -28,18 +28,13 @@ class DeepInspect(BackdoorDefense):
     @classmethod
     def add_argument(cls, group: argparse._ArgumentGroup):
         super().add_argument(group)
-        group.add_argument('--sample_ratio', dest='sample_ratio', type=float,
-                           help='sample ratio from the full training data')
-        group.add_argument('--noise_dim', dest='noise_dim', type=int,
-                           help='GAN noise dimension')
-        group.add_argument('--remask_epoch', dest='remask_epoch', type=int,
-                           help='Remask optimizing epoch')
-        group.add_argument('--remask_lr', dest='remask_lr', type=float,
-                           help='Remask optimizing learning rate')
-        group.add_argument('--gamma_1', dest='gamma_1', type=float,
-                           help='control effect of GAN loss')
-        group.add_argument('--gamma_2', dest='gamma_2', type=float,
-                           help='control effect of perturbation loss')
+        group.add_argument('--sample_ratio', type=float, help='sample ratio from the full training data')
+        group.add_argument('--noise_dim', type=int, help='GAN noise dimension')
+        group.add_argument('--remask_epoch', type=int, help='Remask optimizing epoch')
+        group.add_argument('--remask_lr', type=float, help='Remask optimizing learning rate')
+        group.add_argument('--gamma_1', type=float, help='control effect of GAN loss')
+        group.add_argument('--gamma_2', type=float, help='control effect of perturbation loss')
+        return group
 
     def __init__(self, sample_ratio: float = 0.1, noise_dim: int = 100,
                  remask_epoch: int = 30, remask_lr=0.01,
@@ -57,7 +52,7 @@ class DeepInspect(BackdoorDefense):
         self.noise_dim: int = noise_dim
 
         dataset = self.dataset.get_dataset(mode='train')
-        subset, _ = self.dataset.split_set(dataset, percent=sample_ratio)
+        subset, _ = self.dataset.split_dataset(dataset, percent=sample_ratio)
         self.loader = self.dataset.get_dataloader(mode='train', dataset=subset)
 
     def detect(self, **kwargs):
@@ -66,7 +61,7 @@ class DeepInspect(BackdoorDefense):
             self.real_mask = self.attack.mark.mask
         loss_list, mark_list = self.get_potential_triggers()
         np.savez(os.path.join(self.folder_path, self.get_filename(target_class=self.target_class) + '.npz'),
-                 mark_list=mark_list, loss_list=loss_list)
+                 mark_list=to_numpy(mark_list), loss_list=to_numpy(loss_list))
         print('loss: ', loss_list)
         print('loss MAD: ', normalize_mad(loss_list))
 
@@ -79,6 +74,7 @@ class DeepInspect(BackdoorDefense):
             loss_list.append(loss)
             mark_list.append(mark)
         loss_list = torch.as_tensor(loss_list)
+        mark_list = torch.stack(mark_list)
         return loss_list, mark_list
 
     def load(self, path: str = None):
@@ -94,7 +90,7 @@ class DeepInspect(BackdoorDefense):
             return _input + self.attack.mark.mark.to(_input.device)
         self.attack.mark.add_mark_fn = add_mark_fn
 
-    def remask(self, label: int) -> tuple[torch.Tensor, torch.Tensor]:
+    def remask(self, label: int) -> tuple[float, torch.Tensor]:
         generator = Generator(self.noise_dim, self.dataset.num_classes, self.dataset.data_shape)
         for param in generator.parameters():
             param.requires_grad_()
@@ -203,7 +199,8 @@ class Generator(nn.Module):
             self.cuda()
 
     def forward(self, noise: torch.Tensor, poison_label: torch.Tensor) -> torch.Tensor:
-        _label = onehot_label(poison_label, self.num_classes).float()
+        _label: torch.Tensor = F.one_hot(poison_label, self.num_classes)
+        _label = _label.float()
         y_ = self.fc2(_label)
         y_ = self.relu(y_)
         x = torch.cat([noise, y_], dim=1)
