@@ -2,12 +2,16 @@
 
 from ..backdoor_defense import BackdoorDefense
 from trojanvision.environ import env
+from trojanzoo.utils.data import dataset_to_list
 
 import torch
-import torch.utils.data
 from torch.utils.data import TensorDataset
 import argparse
 
+from typing import TYPE_CHECKING
+if TYPE_CHECKING:
+    import torch.utils.data
+
 
 class SpectralSignature(BackdoorDefense):
 
@@ -36,59 +40,40 @@ class SpectralSignature(BackdoorDefense):
     @classmethod
     def add_argument(cls, group: argparse._ArgumentGroup):
         super().add_argument(group)
-        group.add_argument('--preprocess_layer', dest='preprocess_layer', type=str,
+        group.add_argument('--preprocess_layer',
                            help='the chosen layer used to extract feature representation, defaults to ``flatten``')
-        group.add_argument('--poison_image_num', dest='poison_image_num', type=int,
+        group.add_argument('--poison_image_num', type=int,
                            help='the number of sampled poison image to train the model initially, defaults to 50')
-        group.add_argument('--clean_image_num', dest='clean_image_num', type=int,
+        group.add_argument('--clean_image_num', type=int,
                            help='the number of sampled clean image to train the model initially, defaults to 500')
-        group.add_argument('--epsilon', dest='epsilon', type=int,
+        group.add_argument('--epsilon', type=int,
                            help='the number of examples to remove from each class, defaults to 5')
-        group.add_argument('--retrain_epoch', dest='retrain_epoch', type=int,
+        group.add_argument('--retrain_epoch', type=int,
                            help='the epoch to retrain the model on clean image dataset, defaults to 5')
+        return group
 
-    def __init__(self, poison_image_num: int = 50, clean_image_num: int = 500, preprocess_layer: str = 'features', epsilon: int = 5, retrain_epoch: int = 5, **kwargs):
+    def __init__(self, poison_image_num: int = 50, clean_image_num: int = 500, preprocess_layer: str = 'flatten', epsilon: int = 5, retrain_epoch: int = 5, **kwargs):
         super().__init__(**kwargs)
         self.preprocess_layer: str = preprocess_layer
         self.poison_image_num: int = poison_image_num
         self.clean_image_num: int = clean_image_num
         self.epsilon: int = epsilon
         self.retrain_epoch: int = retrain_epoch
+        self.param_list['spectral_signature'] = ['preprocess_layer', 'poison_image_num', 'clean_image_num',
+                                                 ' epsilon', 'retrain_epoch']
 
-        self.clean_dataset, _ = self.dataset.split_set(
+        clean_dataset, remain_dataset = self.dataset.split_dataset(
             dataset=self.dataset.get_full_dataset(mode='train'), length=self.clean_image_num)
-        label_all = torch.empty([])    # TODO
-        clean_input_all = torch.empty([])    # TODO
-        for i, data in enumerate(iter(self.clean_dataset)):
-            _input, _label = self.model.get_data(data)
-            clean_input = _input.view(1, _input.shape[0], _input.shape[1], _input.shape[2])
-            if i == 0:
-                clean_input_all = clean_input
-                label_all = torch.unsqueeze(_label, 0)
-            else:
-                clean_input_all = torch.cat((clean_input_all, clean_input))
-                label_all = torch.cat((label_all, torch.unsqueeze(_label, 0)))
-        label_all = torch.squeeze(label_all, 0)
-        self.clean_dataset = TensorDataset(clean_input_all, label_all)
-        self.clean_dataloader = self.dataset.get_dataloader(mode='train', dataset=self.clean_dataset, num_workers=0)
+        clean_input, clean_label = dataset_to_list(clean_dataset)
+        clean_input = torch.stack(clean_input)
+        clean_label = torch.as_tensor(clean_label, dtype=torch.long)
+        self.clean_dataset = TensorDataset(clean_input, clean_label)
 
-        self.poison_dataset, _ = self.dataset.split_set(dataset=_, length=self.poison_image_num)
-        label_all = torch.empty([])    # TODO
-        poison_input_all = torch.empty([])    # TODO
-        for i, data in enumerate(iter(self.poison_dataset)):
-            _input, _label = self.model.get_data(data)
-            poison_input = self.attack.add_mark(_input)
-            poison_input = poison_input.view(1, poison_input.shape[0], poison_input.shape[1], poison_input.shape[2])
-            if i == 0:
-                poison_input_all = poison_input
-                label_all = torch.unsqueeze(_label, 0)
-            else:
-                poison_input_all = torch.cat((poison_input_all, poison_input))
-                label_all = torch.cat((label_all, torch.unsqueeze(_label, 0)))
-        label_all = torch.squeeze(label_all, 0)
-        self.poison_dataset = TensorDataset(poison_input_all, label_all)
-        self.poison_dataloader = self.dataset.get_dataloader(
-            mode='train', dataset=self.poison_dataset, num_workers=0, pin_memory=False)
+        poison_dataset, _ = self.dataset.split_dataset(dataset=remain_dataset, length=self.poison_image_num)
+        poison_input, poison_label = dataset_to_list(poison_dataset)
+        poison_input = torch.stack(poison_input)
+        poison_label = torch.as_tensor(poison_label, dtype=torch.long)
+        self.poison_dataset = TensorDataset(poison_input, poison_label)
 
         self.mix_dataset = torch.utils.data.ConcatDataset([self.clean_dataset, self.poison_dataset])
         self.mix_dataloader = self.dataset.get_dataloader(
@@ -118,43 +103,44 @@ class SpectralSignature(BackdoorDefense):
         """
         final_set = None    # TODO
         for k in range(self.dataset.num_classes):
-            # self.class_dataset = self.dataset.get_class_set(self.mix_dataset,classes = [k])
+            # class_dataset = self.dataset.get_class_subset(self.mix_dataset,classes = [k])
             idx = []
             for i, data in enumerate(self.mix_dataset):
                 _input, _label = self.model.get_data(data)
                 _input = _input.view(1, _input.shape[0], _input.shape[1], _input.shape[2])
                 if _label.item() == k:
                     idx.append(k)
-            self.class_dataset = torch.utils.data.Subset(self.mix_dataset, idx)
+            class_dataset = torch.utils.data.Subset(self.mix_dataset, idx)
+            class_input, class_label = dataset_to_list(class_dataset)
+            class_input = torch.stack(class_input)
+            class_label = torch.as_tensor(class_label, dtype=torch.long)
+            class_dataset = TensorDataset(class_input, class_label)
+            class_dataloader = self.dataset.get_dataloader(mode='train', dataset=class_dataset, num_workers=0)
 
-            layer_output_all = torch.empty([])    # TODO
-            for i, data in enumerate(self.class_dataset):
+            layer_output_all = []   # TODO
+            for i, data in enumerate(class_dataloader):
                 _input, _label = self.model.get_data(data)
                 layer_output = self.model.get_layer(_input, layer_output=self.preprocess_layer)
-                layer_output = layer_output.view(1, -1)
-                if i == 0:
-                    layer_output_all = layer_output
-                else:
-                    layer_output_all = torch.cat((layer_output_all, layer_output))
-
+                layer_output_all.append(layer_output.flatten(1))
+            layer_output_all = torch.cat(layer_output_all, dim=0)
             layer_output_mean = torch.mean(layer_output_all, dim=0)
 
-            for i in range(len(self.class_dataset)):
+            for i in range(len(class_dataset)):
                 layer_output_all[i] = layer_output_all[i] - layer_output_mean
 
             u, s, v = torch.svd(layer_output_all)
-            v_transpose = torch.transpose(v, 1, 0)
+            vt = v.transpose(0, 1)
             outlier_scores = torch.rand([layer_output_all.shape[0]], device=env['device'])
-            for i in range(len(self.class_dataset)):
-                outlier_scores[i] = (layer_output_all[i].view(1, -1) @ v_transpose[i].view(-1, 1)) ** 2
+            for i in range(len(class_dataset)):
+                outlier_scores[i] = (layer_output_all[i].view(1, -1) @ vt[i].view(-1, 1)) ** 2
             outlier_scores_sorted, indices = torch.sort(outlier_scores, descending=True)
 
             clean_indices = indices[self.epsilon:]
-            self.class_dataset = torch.utils.data.Subset(self.class_dataset, clean_indices)
+            class_dataset = torch.utils.data.Subset(class_dataset, clean_indices)
             if k == 0:
-                final_set = self.class_dataset
+                final_set = class_dataset
             else:
-                final_set = torch.utils.data.ConcatDataset([final_set, self.class_dataset])
+                final_set = torch.utils.data.ConcatDataset([final_set, class_dataset])
 
         final_dataloader = self.dataset.get_dataloader(mode=None, dataset=final_set, num_workers=0, pin_memory=False)
         return final_dataloader
