@@ -2,7 +2,7 @@
 
 from .output import ansi
 from .param import Param
-from .miscellaneous import get_name
+from .others import get_name
 from trojanzoo.configs import config
 
 import torch
@@ -15,34 +15,31 @@ from typing import Union    # TODO: python 3.10
 from trojanzoo.configs import Config
 import argparse
 if TYPE_CHECKING:
-    import torch.cuda
     import torch.backends.cudnn
 
 
 class Env(Param):
 
-    @staticmethod
-    def add_argument(group: argparse._ArgumentGroup):
+    @classmethod
+    def add_argument(cls, group: argparse._ArgumentGroup):
         group.add_argument('--config', dest='config_path',
                            help='cmd config file path. (``package < project < cmd_config < cmd_param``)')
 
-        group.add_argument('--seed', dest='seed', type=int,
+        group.add_argument('--seed', type=int,
                            help='the random seed for numpy, torch and cuda, defaults to config[env][seed]=1228')
-        group.add_argument('--cache_threshold', dest='cache_threshold', type=float,
+        group.add_argument('--cache_threshold', type=float,
                            help='the threshold (MB) to call torch.cuda.empty_cache(), defaults to config[env][cache_threshold]=None (never).')
 
-        group.add_argument('--device', dest='device',
-                           help='set to \'cpu\' to force cpu-only and \'gpu\', \'cuda\' for gpu-only, defaults to None.')
-        group.add_argument('--benchmark', dest='benchmark', action='store_true',
+        group.add_argument('--device', help='set to "cpu" to force cpu-only '
+                           'and "gpu", "cuda" for gpu-only, defaults to None.')
+        group.add_argument('--benchmark', action='store_true',
                            help='use torch.backends.cudnn.benchmark to accelerate without deterministic, defaults to False.')
-        group.add_argument('--verbose', dest='verbose', type=int, default=0,
-                           help='show arguments and module information, defaults to False.')
-        group.add_argument('--color', dest='color', action='store_true',
-                           help='Colorful Output, defaults to False.')
-        group.add_argument('--tqdm', dest='tqdm', action='store_true',
-                           help='Show tqdm Progress Bar, defaults to False.')
+        group.add_argument('--verbose', type=int, default=0,
+                           help='show arguments and module information, defaults to 0.')
+        group.add_argument('--color', action='store_true', help='colorful Output.')
+        group.add_argument('--tqdm', action='store_true', help='show tqdm progress bar.')
 
-        # group.add_argument('--world_size', dest='world_size', type=int, default=1)
+        # group.add_argument('--world_size', type=int, default=1)
         return group
 
 
@@ -64,7 +61,7 @@ def create(config_path: str = None, dataset_name: str = None, dataset: str = Non
     config.update_cmd(config_path)
     dataset_name = get_name(name=dataset_name, module=dataset, arg_list=['-d', '--dataset'])
     dataset_name = dataset_name if dataset_name is not None else config.get_full_config()['dataset']['default_dataset']
-    result = config.get_config(dataset_name=dataset_name)['env']._update(other_kwargs)
+    result = config.get_config(dataset_name=dataset_name)['env'].update(other_kwargs)
     env.update(config_path=config_path, **result)
     ansi.switch(env['color'])
     if seed is None and 'seed' in env.keys():
@@ -100,4 +97,7 @@ def create(config_path: str = None, dataset_name: str = None, dataset: str = Non
     return env
 
 
+create()
+
+
 # def init_distributed_mode():
