@@ -3,39 +3,38 @@
 from .output import prints
 
 from typing import TYPE_CHECKING
-from typing import Any, Generic, Mapping, MutableMapping, Iterator, TypeVar    # TODO: python 3.10
+from typing import Generic, MutableMapping, TypeVar
 _KT = TypeVar("_KT")  # Key type.
 _VT = TypeVar("_VT")  # Value type.
-_T_co = TypeVar("_T_co", covariant=True)  # Any type covariant containers.
 if TYPE_CHECKING:
     pass
 
 
-class Module(MutableMapping[_KT, _VT], Generic[_KT, _VT]):
+class Module(MutableMapping[_KT, _VT], Generic[_KT, _VT]):  # TODO: issue 3 why need Generic
     _marker = 'M'
 
-    def __init__(self, *args: list[Mapping[_KT, _VT]], **kwargs):
+    def __init__(self, *args: MutableMapping[_KT, _VT], **kwargs: _VT):
         self.__data: dict[_KT, _VT] = {}
         if len(args) == 1 and args[0] is None:
             return
         self.update(*args, **kwargs)
 
-    def update(self, *args: list[Mapping[_KT, _VT]], **kwargs):
-        args = list(args)
+    def update(self, *args: MutableMapping[_KT, _VT], **kwargs: _VT):
+        args: list = list(args)     # TODO: issue 2 pylance issue
         args.append(kwargs)
         for module in args:
             self._update(module)
         return self
 
-    def _update(self, module: Mapping[_KT, _VT]):    # TODO: Union[dict, Module]
+    def _update(self, module: MutableMapping[_KT, _VT]):    # TODO: issue 4 Union[dict, Module]
         for key, value in module.items():
             if value is None:
                 continue
             if key in self.keys() and isinstance(self[key], Module):
-                self[key].update(value)     # TODO: linting problem
+                sub_module: Module = self[key]
+                sub_module.update(value)
             elif key in self.keys() and isinstance(value, Module):
-                self[key] = type(value)(self[key])
-                self[key].update(value)
+                self[key] = type(value)(self[key]).update(value)
             elif isinstance(value, Module):
                 self[key] = value.copy()
             else:
@@ -43,12 +42,8 @@ class Module(MutableMapping[_KT, _VT], Generic[_KT, _VT]):
         return self
 
     def remove_none(self):
-        """Remove the parameters whose values are ``None``
-
-        :return: ``self``
-        :rtype: Module
-        """
-        for key in list(self.__data.keys()):
+        """Remove the parameters whose values are ``None``"""
+        for key in self.__data.keys():
             if self.__data[key] is None:
                 del self.__data[key]
         return self
@@ -57,7 +52,7 @@ class Module(MutableMapping[_KT, _VT], Generic[_KT, _VT]):
         return type(self)(self)
 
     def clear(self):
-        for item in list(self.keys()):
+        for item in self.keys():
             delattr(self, item)
         return self
 
@@ -69,7 +64,7 @@ class Module(MutableMapping[_KT, _VT], Generic[_KT, _VT]):
 
     def __getattr__(self, name: _KT) -> _VT:
         if name == '_Module__data':
-            super().__getattr__(name)
+            return super().__getattr__(name)
         return self.__data[name]
 
     def __getitem__(self, k: _KT):
@@ -95,27 +90,26 @@ class Module(MutableMapping[_KT, _VT], Generic[_KT, _VT]):
     def __repr__(self):
         return self._marker + self.__data.__repr__()
 
-    def __len__(self) -> int:
+    def __len__(self):
         return self.__data.__len__()
 
-    def __iter__(self) -> Iterator[_T_co]:
+    def __iter__(self):
         return self.__data.__iter__()
 
     def summary(self, indent: int = 0):
         prints(self, indent=indent)
 
 
-class Param(Module, Generic[_KT, _VT]):
+class Param(Module, Generic[_KT, _VT]):  # TODO: issue 3 why need Generic, Module[_KT, _VT]
     _marker = 'P'
 
-    def update(self, *args: list[dict], **kwargs):
-        if len(kwargs) == 0 and len(args) == 1 and not isinstance(args[0], dict) \
-                and not isinstance(args[0], Module):
+    def update(self, *args: dict[_KT, _VT], **kwargs: _VT):
+        if len(kwargs) == 0 and len(args) == 1 and not isinstance(args[0], (dict, Module)):
             self.default = args[0]
             return self
         return super().update(*args, **kwargs)
 
-    def _update(self, module: dict[str, Any]):
+    def _update(self, module: dict[_KT, _VT]):
         for key, value in module.items():
             if key == 'default':
                 self.default = value
@@ -136,8 +130,10 @@ class Param(Module, Generic[_KT, _VT]):
     def __getattr__(self, name: str) -> _VT:
         try:
             return super().__getattr__(name)
-        except KeyError:    # TODO: or AttributeError better?
-            return self['default']
+        except KeyError as e:
+            if 'default' in self.keys():
+                return self['default']
+            raise e
 
     def __getitem__(self, key: str) -> _VT:
         if key not in self.keys():
